#!/bin/sh

# Shell script to handle SCC web server actions.
# Copyright (C) 2001-2004 Open Challenge B.V.
# Copyright (C) 2004-2005 OpenEyeT Professional Services.
# Copyright (C) 2005-2009 QNH.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.
# If not, write to the Free Software Foundation,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Contact information: http://scc.qnh-infrastructure.nl

# SCC-release:	1.7.5
# file-version:	$Revision: 1.81 $


ProgName=${0##*/};			export ProgName

set -u

SCC_BIN=/opt/scc-srv/bin
SCC_TOP=/var/opt/scc-srv
export SCC_BIN SCC_TOP

PATH=/sbin:/usr/sbin:/usr/bin:/bin:${SCC_BIN};		export PATH

export SCC_DATA=${SCC_TOP}/data
export SCC_TMP=${SCC_TOP}/tmp
export SCC_CONF=${SCC_TOP}/conf
export SCC_WWW=${SCC_DATA}/www

export SCC_LOG=${SCC_DATA}/log/scc.cgi.log
logging=
if [ -w ${SCC_LOG} ]
then
	logging="yes"
fi
log_date=$(date '+%Y-%m-%d:%H.%M.%S')

export TMPDIR=${SCC_TMP}
export TMP=${SCC_TMP}

export SHELL=/bin/sh
export LANG=C

# Use the following utilities for SunOS.
if [ -x /usr/ccs/bin/what ]
then
	alias what=/usr/ccs/bin/what
fi
if [ -x /usr/xpg4/bin/awk ]
then
	alias awk=/usr/xpg4/bin/awk
fi
if [ -x /usr/xpg4/bin/grep ]
then
	alias grep=/usr/xpg4/bin/grep
fi
if [ -x /usr/xpg4/bin/tail ]
then
	alias tail=/usr/xpg4/bin/tail
fi

SYNTAX="Syntax error, use: ${ProgName} <realm>"

if [ $# -ne 1 ]
then
	echo "${SYNTAX}" >&2
	exit 1
fi
realm="${1}"

scc_web_path_dir=""
if [ -f ${SCC_CONF}/scc.conf ]
then
	scc_web_path_dir="$(sed -n 's/^SCC_WEB_PATH=//p' ${SCC_CONF}/scc.conf)"
fi
web_realm_dir="${scc_web_path_dir}/${realm}"
HOME_URL="<A HREF=\"${web_realm_dir}/index.html\">Home</A>&nbsp;&nbsp;&nbsp;&nbsp;"

bg_color=""
if [ -f ./custom/scc-realm.conf ]
then
	color="$(sed -n 's/^BG_COLOR=//p' ./custom/scc-realm.conf)"
	if [ "${color}" ]
	then
		bg_color=" bgcolor=\"${color}\""
	fi
fi

logo_tag=""
if [ -f ./custom/scc-logo.png ]
then
	logo_tag="<IMG SRC=\"${web_realm_dir}/custom/scc-logo.png\" ALT=HelpInfo BORDER=0>"
fi

group=${SCC_TMP}/scc-cgi-class-$$;		export group
TMP_FILE_1=${SCC_TMP}/scc-cgi-tmp1-$$;		export TMP_FILE_1
TMP_FILE_2=${SCC_TMP}/scc-cgi-tmp2-$$;		export TMP_FILE_2

trap "rm -f ${group} ${TMP_FILE_1} ${TMP_FILE_2}" 0
trap "exit 2" 1 2 3 15

header()
{
	cat <<-EndOfTxt
		<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
		<HTML>
		<HEAD>
			<LINK HREF="${web_realm_dir}/custom/style.css" REL="stylesheet" TYPE="text/css">
			<TITLE>System Configuration Collector: ${realm}</TITLE>
		</HEAD>
		<BODY ${bg_color}>
		<CENTER>
	EndOfTxt
}

trailer()
{
	echo "</CENTER>"
	echo "</BODY>"
	echo "</HTML>"
}

# We are called as a CGI-script, produce the proper heading.
echo "Content-Type: text/html;charset=iso-8859-1"
echo ""

# Without using the -w option of scc-update, several actions fail with unclear errors.
# Check and report whether the -w option has been used (indicated by Ben Jannedy).
if [ ! -w "${SCC_TMP}" ]
then
	header
	echo "<H4>Insufficient access to tmp-directory of scc-srv, use: scc-update -w &lt;webuser&gt;</H4>"
	trailer
	exit 0
fi

# Maarten Hartsuijker pointed us at the following:
# replace '<' (%3c) and '>' (%3e) by '_' to avoid Cross Site Scripting.
QS=$(echo "${QUERY_STRING}"			|
	sed	-e 's/+/ /g'		\
		-e 's/%23/#/g'		\
		-e 's/%2[Aa]/*/g'	\
		-e 's/%2[Bb]/+/g'	\
		-e 's/%2[Dd]/-/g'	\
		-e 's/%2[Ee]/./g'	\
		-e 's/%2[Ff]/\//g'	\
		-e 's/%3[Aa]/:/g'	\
		-e 's/%3[Bb]/;/g'	\
		-e 's/%3[Dd]/=/g'	\
		-e 's/%5[Bb]/[/g'	\
		-e 's/%5[Cc]/\\/g'	\
		-e 's/%5[Dd]/]/g'	\
		-e 's/%5[Ee]/^/g'	\
		-e 's/%5[Ff]/_/g'	\
		-e 's/%3[Cc]/_/g'	\
		-e 's/</_/g'		\
		-e 's/%3[Ee]/_/g'	\
		-e 's/>/_/g'		\
		-e 's/%//g')

# Using the first START-button, to retrieve data, results in the following
# format for QUERY_STRING: sys=<system>&opt=<option>&search=<search>&s_case=<case>
# Start to check whether option is non-empty. Determine other variables later.
option=$(echo "${QS}" | sed -n -e 's/.*opt=//p' | sed -e 's/&.*//')

if [ -n "${option}" ]
then
	system=$(echo "${QS}" | sed -n -e 's/sys=//p' | sed -e 's/&.*//')
	string=$(echo "${QS}" | sed -n -e 's/.*search=//p' | sed -e 's/&.*//')
	s_case=$(echo "${QS}" | sed -n -e 's/.*s_case=//p' | sed -e 's/&.*//')

	if [ -n "${string}" ]
	then
		# A search-string has been supplied, we are going to search.
		header
		echo "<A HREF=\"${scc_web_path_dir}/scc-help/scc-search-index.html\">${logo_tag}</A>"

		case_opt="-i"
		case_label="(ignore case)"
		if [ -n "${s_case}" ]
		then
			case_opt=""
			case_label="(exact match)"
		fi

		search_files="snapshot(s)"
		search_suffix="cur"
		show_suffix=""
		if [ ${option} = "log" ]
		then
			search_files="logbook(s)"
			search_suffix="log"
			show_suffix=".log"
		elif [ ${option} = "smt" ]
		then
			search_files="SMT-data"
			show_suffix=".smt"
		elif [ ${option} = "rules" ]
		then
			search_files="Rules"
			show_suffix=""
		fi

		[ "${logging}" ] && echo "${log_date}:${realm}:search for:${string}: in ${system}, ${search_files}" >>${SCC_LOG}

		SEARCH_LABEL="Searched for: &quot;<font color=red>${string}</font>&quot; ${case_label} in ${search_files} for ${system}"

		echo "<H4>"
		echo "${HOME_URL}"
		echo "${SEARCH_LABEL}"
		echo "</H4>"

		if [ "${system}" = "all_systems" ]
		then
			search_sys=""

			# Use scc-summary.data to determine all systems, format is:
			# "general_data":<host>:<model>:<os>:<release>:<last_day>:<last_time>
			# File is sorted on hostname (ignore case).
			search_files="$(sed	-e "s/^general_data:/scc./"	\
						-e "s/:.*/.${search_suffix}/" scc-summary.data)"

			count_all=$(wc -l <scc-summary.data)
		else
			search_sys="${system}"
			search_files="scc.${system}.${search_suffix}"
			count_all=1
		fi

		# Restrict the amount of data per system to the limit:
		# - 10 lines for multi-file search
		# - 1000 lines for single file search
		# We allow one extra line to signal the "overflow" in the result of the search.
		limit=1001
		if [ ${count_all} -gt 1 ]
		then
			limit=11
		fi

		rm -f ${TMP_FILE_1} ${TMP_FILE_2}

		if [ ${option} = "smt" ]
		then
			awk -F:	'{
					# Format of the input-data is:
					# smt-data:<host>:<software>:<version>
					if ( ( s != "" ) && ( s != $2 ) )
					{
						next;
					}
					$1 = "";
					if ( length( c ) > 0 )
					{
						# Ignore case, map data and RE to uppercase.
						m = match( toupper( $0 ), toupper( r ) );
					}
					else
					{
						m = match( $0, r );
					}
					if ( m > 0 )
					{
						sys = $2;
						$2 = "";
						printf( "%s %s\n", sys, $0 );
					}
				}'	c="${case_opt}"		\
					s="${search_sys}"	\
					r="${string}"		\
						scc-smt.data
		elif [ ${option} = "rules" ]
		then
			awk	'/^#/	{ next }
					{
						# Format of the input-data is:
						# <keyword> <host> <data>
						if ( ( s != "" ) && ( s != $2 ) )
						{
							next;
						}
						if ( length( c ) > 0 )
						{
							# Ignore case, map data and RE to uppercase.
							m = match( toupper( $0 ), toupper( r ) );
						}
						else
						{
							m = match( $0, r );
						}
						if ( m > 0 )
						{
							sys = $2;
							$2 = "";
							printf( "%s %s\n", sys, $0 );
						}
					}'	c="${case_opt}"		\
						s="${search_sys}"	\
						r="${string}"		\
							scc-rules.data
		elif [ ${option} = "log" ]
		then
			echo "${search_files}"								|
			xargs grep ${case_opt} "${string}" /dev/null					|
			sort -t: -k1,1f -k2,3r								|
			sed	-e 's/^scc\.//'	\
				-e 's/\.log:/ /'
		else
			echo "${search_files}"								|
			xargs grep ${case_opt} "${string}" /dev/null					|
			sed	-e 's/^scc\.//'	\
				-e 's/\.cur:/ /'
		fi											|
		awk 	'{
				# We count the number of hits for each system, format of the data is:
				# <host> <data>
				cnt[ $1 ] += 1;
				if ( cnt[ $1 ] <= l )
				{
					print;
				}
			}
			END	{
					for ( s in cnt )
					{
						print "cnt", s, cnt[ s ] >>o;
					}
				}'	l="${limit}"	\
					o=${TMP_FILE_2}			|
		sed	-e 's/&/\&amp;/g'	\
			-e 's/</\&lt;/g'	\
			-e 's/>/\&gt;/g'	\
			-e 's/"/\&quot;/g' >${TMP_FILE_1}

		systems_hit=$(sed -e 's/^cnt://' -e 's/:.*//' ${TMP_FILE_2} | sort -f )
		if [ -z "${systems_hit}" ]
		then
			echo "<H4>No match found.</H4>"
			trailer
			exit 0
		fi
		count_hit=$(wc -l <${TMP_FILE_2})
		absent=$(( ${count_all} - ${count_hit} ))

		if [ "${absent}" -eq 0 ]
		then
			echo "<P>Data found on all systems.</P>"
		else
			echo "<P>Click <A HREF=\"#absent\">here</A> for ${absent} system(s) without a match."
			echo "The following ${count_hit} system(s) matched:</P>"
		fi

		echo "<TABLE CLASS=SCC BORDER SUMMARY=\"Search results\" CELLSPACING=1 CELLPADDING=4>"

		echo "<THEAD>"
		echo "	<TR class=Odd align=left>"
		echo "		<TH>System</TH>"
		echo "		<TH>Hits</TH>"
		echo "		<TH>Data</TH>"
		echo "	</TR>"
		echo "</THEAD>"

		echo "<TBODY>"

		# All data from one system (including a possible "overflow" message) are displayed
		# as a single cell of a table.
		awk	'BEGIN	{
					c[ 0 ] = "Even";
					c[ 1 ] = "Odd";
					tr_c = 2;		# TR class 
				}
			/^cnt/	{ cnt[ $2 ] = $3; next }
				{
					# Format of the data is:
					# <host> <data>
					if ( $1 != prev )
					{
						if ( data_shown )
						{
							# terminate row of previous system
							printf( "    </TD>\n</TR>\n" );
						}

						host_html = sprintf("<A HREF=\"%s/%s/scc.%s%s.html\">%s</A>",\
									d, r, $1, s, $1 );
						prev = $1;
						cnt_per_host = 0;
					}
					else
					{
						host_html = "";
					}
					data_shown = 1;

					cnt_per_host++;
					if ( cnt_per_host < l )
					{
						if ( length( host_html ) )
						{
							# Keep hostname and hit-count at first line of cell.
							# Do not wrap the data and fqdn-hostnames.
							printf( "<TR class=%s><TD class=Even nowrap valign=top>%s</TD>\n",\
								c[ tr_c++ % 2 ], host_html );
							printf( "	<TD class=Odd valign=top>%d</TD>\n",\
								cnt[ $1 ] );
							printf( "	<TD class=Even nowrap>    <TT>" );
						}
						else
						{
							printf( "	<BR><TT>" );
						}
						$1 = "";
						sub( "^  *", "", $0 );
						print $0 "</TT>";
					}
					else if ( cnt_per_host == l )
					{
						print "	<BR><font color=red>data limited to " l - 1	\
							" lines per system</font>"
					}
				}
				END	{
						if ( NR > 0 )
						{
							printf( "    </TD>\n</TR>\n" );	# Terminate the last cell
						}
						printf( "</TBODY>\n" );
						printf( "</TABLE>\n" );			# End the table
					}'	l="${limit}"			\
						s="${show_suffix}"		\
						d="${scc_web_path_dir}"		\
						r="${realm}"			\
							${TMP_FILE_2} ${TMP_FILE_1}

		if [ "${absent}" -ne 0 ]
		then
			echo "<A NAME=absent></A>"
			echo "<H4>"
			echo "<A HREF=\"#cfg_top\">Top</A>&nbsp;&nbsp;&nbsp;&nbsp;"
			echo "${SEARCH_LABEL}"
			echo "</H4>"

			echo "<P>The following ${absent} system(s) did not match:</P>"

			echo "${systems_hit}"			|
			tr " " "\012"				|
			sed -e 's@^@absent:@' >${TMP_FILE_1}

			echo "${search_files}"			|
			tr " " "\012"				|
			sed	-e 's@^scc\.@@'	\
				-e "s@\.${search_suffix}@@"	|
			awk -F: 'BEGIN	{
						c[ 0 ] = "Even";
						c[ 1 ] = "Odd";
						tr_c = 2;		# TR class 
					}
				/^general_data/	{
						# Add some general system info to the systems without a match.
						# Format of the input is:
						# "general_data":<host>:<model>:<os>:<release>:<last_day>:<last_time>
						if ( NF == 7 )
						{
							model[ $2 ] = $3;
							os[ $2 ] = $4;
							rel[ $2 ] = $5;
							last_change[ $2 ]=sprintf( "%s %s", $6, $7 );
						}
						next;
					}
				/^absent/	{ absent[ $2 ] = 1; next;
					}
					{
						if ( absent[ $1 ] > 0 )
						{
							next;
						}

						if ( header_done == 0 )
						{
							print "<TABLE CLASS=SCC BORDER SUMMARY=\"Summary of systems without match\" CELLSPACING=1 CELLPADDING=4>\n";
							print "<THEAD>";
							print "	<TR class=Odd align=left>";
							print "		<TH>System</TH>";
							print "		<TH>Model</TH>";
							print "		<TH>OS</TH>";
							print "		<TH>Last change</TH>";
							print "	</TR>";
							print "</THEAD>";
							print "<TBODY>";
							header_done=1;
						}
						printf( "<TR class=%s>\n", c[ tr_c++ % 2 ] );
						printf( " <TD class=Even><A HREF=\"%s/%s/scc.%s.html\">%s</A></TD>\n", d, r, $1, $1 );
						if ( length( model[ $1 ] ) > 0 )
						{
							printf( " <TD class=Odd>%s</TD>\n", model[ $1 ] );
						}
						else
						{
							printf( " <TD class=Odd>&nbsp;</TD>\n" );
						}
						printf( " <TD class=Even>%s&nbsp;%s</TD>\n", os[ $1 ], rel[ $1 ] );
						if ( length( last_change[ $1 ] ) > 0 )
						{
							printf( " <TD class=Odd><A HREF=\"%s/%s/scc.%s.log.html\">%s</A></TD>\n",\
								d, r, $1, last_change[ $1 ] );
						}
						else
						{
							printf( " <TD class=Odd>&nbsp;</TD>\n" );
						}
						print "</TR>";
					}
					END	{
							print "</TBODY>";
							print "</TABLE>";
						}'	d="${scc_web_path_dir}"	\
							r="${realm}"		\
								scc-summary.data ${TMP_FILE_1} - 2>/dev/null
		fi

		trailer
		exit 0
	fi

	if [ ${option} = "log" ]
	then
		[ "${logging}" ] && echo "${log_date}:${realm}:show logging" >>${SCC_LOG}

		if [ ${system} = "all_systems" ]
		then
			cat "./scc-log-index.html"
		elif [ -r "./scc.${system}.log.html" ]
		then
			# The log file contains an URL to Home (index.html) and an URL to the snapshot.
			# These URLs do not contain a full path. Add the full path to avoid that the files
			# are searched "under" the CGI-script.
			f1="index.html"
			f2="scc.${system}.html"
			sed	-e "s@^<LINK HREF=\"custom/style.css@<LINK HREF=\"${web_realm_dir}/custom/style.css@"	\
				-e "s@HREF=\"*${f1}\"*>Home</A>@HREF=\"${web_realm_dir}/${f1}\">Home</A>@"		\
				-e "s@HREF=\"*${f2}\"*@HREF=\"${web_realm_dir}/${f2}\"@"				\
					< "./scc.${system}.log.html"
		else
			echo "<H4>Cannot read file scc.${system}.html</H4> "
			trailer
		fi
	   	exit 0
	fi

	if [ ${option} = "cnf" ]
	then
		[ "${logging}" ] && echo "${log_date}:${realm}:show snapshots" >>${SCC_LOG}

		if [ ${system} = "all_systems" ]
		then
			cat "./scc-summary-sys.html"
		elif [ -r "./scc.${system}.html" ]
		then
			# The snapshot contains an URL to Home (index.html) and an URL to the log file.
			# These URLs do not contain a full path. Add the full path to avoid that the files
			# are searched "under" the CGI-script.
			f1="index.html"
			f2="scc.${system}.log.html"
			sed	-e "s@^<LINK HREF=\"custom/style.css@<LINK HREF=\"${web_realm_dir}/custom/style.css@"	\
				-e "s@HREF=\"*${f1}\"*>Home</A>@HREF=\"${web_realm_dir}/${f1}\">Home</A>@"		\
				-e "s@HREF=\"*${f2}\"*@HREF=\"${web_realm_dir}/${f2}\"@"				\
					< "./scc.${system}.html"
		else
			echo "<H4>Cannot read file scc.${system}.html</H4> "
			trailer
		fi
		exit 0
	fi

	if [ ${option} = "smt" ]
	then
		[ "${logging}" ] && echo "${log_date}:${realm}:show SMT-data" >>${SCC_LOG}

		if [ ${system} = "all_systems" ]
		then
			if [ -f ./scc-smt-index.html ]
			then
				cat "./scc-smt-index.html"
			else
				echo "<H4>Cannot read file scc-smt-index.html</H4> "
				trailer
			fi
		elif [ -r "./scc.${system}.smt.html" ]
		then
			# The file has been generated by scc-smt and only contains full paths to html-files.
			cat "./scc.${system}.smt.html"
		else
			echo "<H4>Cannot read file scc.${system}.smt.html</H4> "
			trailer
		fi
		exit 0
	fi

	if [ ${option} = "rules" ]
	then
		[ "${logging}" ] && echo "${log_date}:${realm}:show Rules-data" >>${SCC_LOG}

		if [ ${system} = "all_systems" ]
		then
			if [ -f ./scc-rules-index.html ]
			then
				cat "./scc-rules-index.html"
			else
				echo "<H4>Cannot read file scc-rules-index.html</H4> "
				trailer
			fi
		else
			# Get the data for the system, preserve the headers related to the messages.
			awk	'/^#/	{ header = $0; next }
					{
						if ( $2 == s )
						{
							if ( length( header ) )
							{
								print header;
								header = "";
							}
							print;
						}
					}' s="${system}" scc-rules.data 2>/dev/null		|
			${SCC_BIN}/scc-rules -d "${scc_web_path_dir}" -h ${system} "${realm}"

			echo "<CENTER>"
			trailer
		fi
		exit 0
	fi

	echo "<H4>Unknown option: ${option}<H4>"
	trailer
	exit 0
fi

# Using the second START-button, to compare snapshots of two systems, results in the following
# format for QUERY_STRING: sys1=<system1>&sys2=<system2>&class=<class>&class=<class>...
classes=$(echo "${QS}" | tr "&" "\012" | sed -n -e 's/.*class=//p')
system1=$(echo "${QS}" | sed -n -e 's/sys1=//p' | sed -e 's/&.*//')
system2=$(echo "${QS}" | sed -n -e 's/.*sys2=//p' | sed -e 's/&.*//')

header

echo "<A HREF=\"${scc_web_path_dir}/scc-help/scc-compare-index.html\">${logo_tag}</A>"
echo "</CENTER>"

echo "${classes}" >${group}
show_classes=$(echo "${classes}" | tr "\012" ",")
show_classes="${show_classes%,}"

if [ -z "${classes}" ]
then
	echo "<H4>No class(es) specified</H4>"
elif [ "${system1}" = "${system2}" ]
then 
	echo "<H4>Identical systems are not compared</H4>"
else
	[ "${logging}" ] && echo "${log_date}:${realm}:compare systems:${system1} and ${system2}" >>${SCC_LOG}

	if [ -r scc.${system1}.cur -a -r scc.${system2}.cur ]
	then
		${SCC_BIN}/scc-syscmp -l ${TMP_FILE_2} ${group} scc.${system1}.cur scc.${system2}.cur >${TMP_FILE_1} 
		cnt_1="$(head -n 1 ${TMP_FILE_2})"
		cnt_2="$(tail -n 1 ${TMP_FILE_2})"
		label_1="<span class=\"scc_sys_a\">${system1}</span> (${cnt_1} lines of class-data)"
		label_2="<span class=\"scc_sys_b\">${system2}</span> (${cnt_2} lines of class-data)"

		if [ -s ${TMP_FILE_1} ]
		then
			cnt=$(wc -l <${TMP_FILE_1})

			echo "<H4>"
			echo "${HOME_URL}"
			echo "Configuration"
			echo "&nbsp;&nbsp;&nbsp;&nbsp;"
			echo "<A HREF=\"${web_realm_dir}/scc.${system1}.html\">${system1}</A>"
			echo "&nbsp;&nbsp;&nbsp;&nbsp;"
			echo "<A HREF=\"${web_realm_dir}/scc.${system2}.html\">${system2}</A>"
			echo "</H4>"
			echo "<P>${cnt} differences between ${label_1} and ${label_2} for class: ${show_classes}</P>"

			echo "<PRE>"
			sed	-e 's/&/\&amp;/g'							\
				-e 's/</\&lt;/g'							\
				-e 's/>/\&gt;/g'							\
				-e 's/"/\&quot;/g'							\
				-e "s@^\(${system1} *:.*\)@<span class=\"scc_sys_a\">\1</span>@"	\
				-e "s@^\(${system2} *:.*\)@<span class=\"scc_sys_b\">\1</span>@" ${TMP_FILE_1}
			echo "</PRE>"
			echo "<HR>"
		else
			echo "<H4>"
			echo "${HOME_URL}"
			echo "${label_1} and ${label_2} are identical for class: ${show_classes}"
			echo "</H4>"
		fi
		rm -f ${TMP_FILE_1} ${TMP_FILE_2}
	else
		echo "<H4>Cannot find snapshots for ${system1} and ${system2}</H4>"
	fi
fi

echo "</BODY>"
echo "</HTML>"

exit 0
